package org.dromara.work.service.impl;

import org.dromara.common.core.utils.MapstructUtils;
import org.dromara.common.core.utils.StringUtils;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.dromara.work.domain.bo.TpProdBo;
import org.dromara.work.domain.vo.TpProdVo;
import org.dromara.work.domain.TpProd;
import org.dromara.work.mapper.TpProdMapper;
import org.dromara.work.service.ITpProdService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 商品Service业务层处理
 *
 * @author Maosw
 * @date 2025-03-15
 */
@RequiredArgsConstructor
@Service
public class TpProdServiceImpl implements ITpProdService {

    private final TpProdMapper baseMapper;

    /**
     * 查询商品
     *
     * @param id 主键
     * @return 商品
     */
    @Override
    public TpProdVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 分页查询商品列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 商品分页列表
     */
    @Override
    public TableDataInfo<TpProdVo> queryPageList(TpProdBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<TpProd> lqw = buildQueryWrapper(bo);
        Page<TpProdVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的商品列表
     *
     * @param bo 查询条件
     * @return 商品列表
     */
    @Override
    public List<TpProdVo> queryList(TpProdBo bo) {
        LambdaQueryWrapper<TpProd> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<TpProd> buildQueryWrapper(TpProdBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<TpProd> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(TpProd::getId);
        lqw.eq(StringUtils.isNotBlank(bo.getTitle()), TpProd::getTitle, bo.getTitle());
        lqw.eq(bo.getStatus() != null, TpProd::getStatus, bo.getStatus());
        lqw.between(params.get("beginCreateBy") != null && params.get("endCreateBy") != null,
            TpProd::getCreateBy ,params.get("beginCreateBy"), params.get("endCreateBy"));
        return lqw;
    }

    /**
     * 新增商品
     *
     * @param bo 商品
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(TpProdBo bo) {
        TpProd add = MapstructUtils.convert(bo, TpProd.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改商品
     *
     * @param bo 商品
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(TpProdBo bo) {
        TpProd update = MapstructUtils.convert(bo, TpProd.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(TpProd entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除商品信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
